@@ -306,9 +306,15 @@ class Generator:
         name = self._script.utilities.displayedText(obj)
         if obj.getRole() == pyatspi.ROLE_COMBO_BOX:
             children = self._script.utilities.selectedChildren(obj)
-            if not children and obj.childCount:
-                children = self._script.utilities.selectedChildren(obj[0])
-            children = children or [child for child in obj]
+            if not children:
+                try:
+                    children = self._script.utilities.selectedChildren(obj[0])
+                except:
+                    pass
+            try:
+                children = children or [child for child in obj]
+            except:
+                pass
             names = map(self._script.utilities.displayedText, children)
             names = list(filter(lambda x: x, names))
             if len(names) == 1:
@@ -864,12 +870,11 @@ class Generator:
         if self._script.utilities.isLayoutOnly(obj):
             return []
 
-        try:
-            table = obj.queryTable()
-        except:
+        rows, cols = self._script.utilities.rowAndColumnCount(obj)
+        if rows < 0 or cols < 0:
             return []
 
-        return [messages.tableSize(table.nRows, table.nColumns)]       
+        return [messages.tableSize(rows, cols)]
 
     def _generateTableCellRow(self, obj, **args):
         """Orca has a feature to automatically read an entire row of a table
@@ -996,6 +1001,17 @@ class Generator:
 
         return [displayedText]
 
+    def _generateListItemMarker(self, obj, **args):
+        startOffset = args.get('startOffset', 0)
+        if not (0 <= startOffset <= 1):
+            return []
+
+        listItemMarker = self._script.utilities.getListItemMarkerText(obj)
+        if listItemMarker:
+            return [listItemMarker]
+
+        return []
+
     #####################################################################
     #                                                                   #
     # Tree interface information                                        #
