@@ -343,7 +343,8 @@ class LiveRegionManager:
             # look through all the objects on the page and set/add to
             # politeness overrides.  This only adds live regions with good
             # markup.
-            matches = pyatspi.findAllDescendants(docframe, self.matchLiveRegion)
+            matches = self._script.utilities.findAllDescendants(
+                docframe, self.matchLiveRegion)
             for match in matches:
                 objectid = self._getObjectId(match)
                 self._politenessOverrides[(uri, objectid)] = LIVE_OFF
@@ -360,7 +361,7 @@ class LiveRegionManager:
             self.monitoring = True  
 
     def generateLiveRegionDescription(self, obj, **args):
-        """Used in conjuction with whereAmI to output description and 
+        """Used in conjunction with whereAmI to output description and 
         politeness of the given live region object"""
         objectid = self._getObjectId(obj)
         uri = self._script.bookmarks.getURIKey()
@@ -403,6 +404,13 @@ class LiveRegionManager:
         attrs = self._getAttrDictionary(obj)
         return 'container-live' in attrs
 
+    def _findContainer(self, obj):
+        isContainer = lambda x: self._getAttrDictionary(x).get('atomic')
+        if isContainer(obj):
+            return obj
+
+        return pyatspi.findAncestor(obj, isContainer)
+
     def _getMessage(self, event):
         """Gets the message associated with a given live event."""
         attrs = self._getAttrDictionary(event.source)
@@ -422,9 +430,8 @@ class LiveRegionManager:
             if attrs.get('container-atomic') != 'true':
                 content = event.any_data
             else:
-                text = self._script.utilities.queryNonEmptyText(event.source)
-                if text:
-                    content = text.getText(0, -1)
+                container = self._findContainer(event.source)
+                content = self._script.utilities.expandEOCs(container)
 
         if not content:
             return None
