@@ -172,6 +172,16 @@ class Script(web.Script):
         debug.println(debug.LEVEL_INFO, msg, True)
         default.Script.onDocumentReload(self, event)
 
+    def onExpandedChanged(self, event):
+        """Callback for object:state-changed:expanded accessibility events."""
+
+        if super().onExpandedChanged(event):
+            return
+
+        msg = "GECKO: Passing along event to default script"
+        debug.println(debug.LEVEL_INFO, msg, True)
+        default.Script.onExpandedChanged(self, event)
+
     def onFocus(self, event):
         """Callback for focus: accessibility events."""
 
@@ -182,6 +192,11 @@ class Script(web.Script):
         if self.utilities.isLayoutOnly(event.source):
             return
 
+        if event.source == orca_state.activeWindow:
+            msg = "GECKO: Ignoring event for active window."
+            debug.println(debug.LEVEL_INFO, msg, True)
+            return
+
         # NOTE: This event type is deprecated and Orca should no longer use it.
         # This callback remains just to handle bugs in applications and toolkits
         # in which object:state-changed:focused events are missing. And in the
