@@ -89,7 +89,11 @@ class BrailleGenerator(braille_generator.BrailleGenerator):
                 result.append(messages.IMAGE_MAP_LINK)
 
         elif role not in doNotDisplay:
-            result = super()._generateRoleName(obj, **args)
+            label = self._script.utilities.labelForCellCoordinates(obj)
+            if label:
+                result.append(label)
+            else:
+                result = super()._generateRoleName(obj, **args)
 
         index = args.get('index', 0)
         total = args.get('total', 1)
@@ -128,6 +132,21 @@ class BrailleGenerator(braille_generator.BrailleGenerator):
 
         return super()._generateLabelAndName(obj, **args)
 
+    def _generateDescription(self, obj, **args):
+        if self._script.utilities.preferDescriptionOverName(obj):
+            return []
+
+        return super()._generateDescription(obj, **args)
+
+    def _generateName(self, obj, **args):
+        if self._script.utilities.preferDescriptionOverName(obj):
+            return [obj.description]
+
+        if not self._script.utilities.hasValidName(obj):
+            return []
+
+        return super()._generateName(obj, **args)
+
     def _generateExpandedEOCs(self, obj, **args):
         """Returns the expanded embedded object characters for an object."""
         result = []
@@ -140,6 +159,12 @@ class BrailleGenerator(braille_generator.BrailleGenerator):
 
         return result
 
+    def _generateRealActiveDescendantDisplayedText(self, obj, **args):
+        if not self._script.utilities.inDocumentContent(obj):
+            return super()._generateRealActiveDescendantDisplayedText(obj, **args)
+
+        return self._generateDisplayedText(obj, **args)
+
     def _generateTableCellRow(self, obj, **args):
         if not self._script.inFocusMode():
             return super()._generateTableCellRow(obj, **args)
@@ -149,7 +174,7 @@ class BrailleGenerator(braille_generator.BrailleGenerator):
 
         isRow = lambda x: x and x.getRole() == pyatspi.ROLE_TABLE_ROW
         row = pyatspi.findAncestor(obj, isRow)
-        if row and row.name:
+        if row and row.name and not self._script.utilities.isLayoutOnly(row):
             return self.generate(row, includeContext=False)
 
         return super()._generateTableCellRow(obj, **args)
@@ -194,7 +219,7 @@ class BrailleGenerator(braille_generator.BrailleGenerator):
             return []
 
         result = []
-        contents = self._script.utilities.filterContentsForPresentation(contents, False)
+        contents = self._script.utilities.filterContentsForPresentation(contents, True)
 
         obj, offset = self._script.utilities.getCaretContext(documentFrame=None)
         index = self._script.utilities.findObjectInContents(obj, offset, contents)
