@@ -596,6 +596,9 @@ class Script(default.Script):
                     continue
 
                 obj, startOffset, endOffset, text = content
+                if self.utilities.isLabellingContents(obj):
+                    continue
+
                 eventsynthesizer.scrollIntoView(obj)
                 utterances = self.speechGenerator.generateContents(
                     [content], eliminatePauses=True, priorObj=priorObj)
@@ -1182,7 +1185,8 @@ class Script(default.Script):
         self.utilities.setCaretContext(newFocus, caretOffset)
         self.updateBraille(newFocus)
 
-        if self.utilities.isContentEditableWithEmbeddedObjects(newFocus):
+        if self.utilities.isContentEditableWithEmbeddedObjects(newFocus) \
+           and not (newFocus.getRole() == pyatspi.ROLE_TABLE_CELL and newFocus.name):
             msg = "WEB: New focus %s content editable. Generating line contents." % newFocus
             debug.println(debug.LEVEL_INFO, msg, True)
             contents = self.utilities.getLineContentsAtOffset(newFocus, caretOffset)
@@ -1192,7 +1196,7 @@ class Script(default.Script):
             debug.println(debug.LEVEL_INFO, msg, True)
             contents = self.utilities.getLineContentsAtOffset(newFocus, 0)
             utterances = self.speechGenerator.generateContents(contents)
-        elif self.utilities.lastInputEventWasPageNav():
+        elif self.utilities.lastInputEventWasPageNav() and not self.utilities.getTable(newFocus):
             msg = "WEB: New focus %s was scrolled to. Generating line contents." % newFocus
             debug.println(debug.LEVEL_INFO, msg, True)
             contents = self.utilities.getLineContentsAtOffset(newFocus, caretOffset)
@@ -1461,7 +1465,8 @@ class Script(default.Script):
             return True
 
         if self.utilities.lastInputEventWasPageNav() \
-           and not self.utilities.isLink(event.source):
+           and not self.utilities.isLink(event.source) \
+           and not event.source.getRole() == pyatspi.ROLE_COMBO_BOX:
             msg = "WEB: Event handled: Caret moved due to scrolling"
             debug.println(debug.LEVEL_INFO, msg, True)
             self.utilities.setCaretContext(obj, offset)
@@ -1527,7 +1532,8 @@ class Script(default.Script):
         if not _settingsManager.getSetting('caretNavigationEnabled') \
            or self._inFocusMode or isEditable:
             self.utilities.setCaretContext(event.source, event.detail1)
-            orca.setLocusOfFocus(event, event.source, False)
+            notify = event.source.getState().contains(pyatspi.STATE_FOCUSED)
+            orca.setLocusOfFocus(event, event.source, notify)
             msg = "WEB: Setting locusOfFocus, context to: %s, %i" % \
                   (event.source, event.detail1)
             debug.println(debug.LEVEL_INFO, msg, True)
@@ -1624,6 +1630,13 @@ class Script(default.Script):
         debug.println(debug.LEVEL_INFO, msg, True)
 
         if self.utilities.isZombie(obj):
+            if self.utilities.isSameObject(obj, event.any_data, comparePaths=True, ignoreNames=True):
+                msg = "WEB: Event handled by updating locusOfFocus and context"
+                debug.println(debug.LEVEL_INFO, msg, True)
+                orca.setLocusOfFocus(event, event.any_data, False)
+                self.utilities.setCaretContext(event.any_data, offset)
+                return True
+
             obj, offset = self.utilities.getCaretContext(getZombieReplicant=True)
             if not obj:
                 if self._inFocusMode:
@@ -1644,6 +1657,11 @@ class Script(default.Script):
 
         childRole = event.any_data.getRole()
         if childRole == pyatspi.ROLE_ALERT:
+            if event.any_data == self.utilities.lastQueuedLiveRegion():
+                msg = "WEB: Ignoring %s (is last queued live region)" % event.any_data
+                debug.println(debug.LEVEL_INFO, msg, True)
+                return True
+
             msg = "WEB: Presenting event.any_data"
             debug.println(debug.LEVEL_INFO, msg, True)
             self.presentObject(event.any_data)
@@ -1691,6 +1709,21 @@ class Script(default.Script):
         self._loadingDocumentContent = True
         return True
 
+    def onExpandedChanged(self, event):
+        """Callback for object:state-changed:expanded accessibility events."""
+
+        if self.utilities.isZombie(event.source):
+            msg = "WEB: Event source is Zombie"
+            debug.println(debug.LEVEL_INFO, msg, True)
+            return True
+
+        if not self.utilities.inDocumentContent(event.source):
+            msg = "WEB: Event source is not in document content"
+            debug.println(debug.LEVEL_INFO, msg, True)
+            return False
+
+        return False
+
     def onFocusedChanged(self, event):
         """Callback for object:state-changed:focused accessibility events."""
 
@@ -1745,7 +1778,8 @@ class Script(default.Script):
 
             obj, offset = self.utilities.searchForCaretContext(event.source)
             if obj:
-                notify = self.utilities.inFindToolbar(orca_state.locusOfFocus)
+                notify = self.utilities.inFindToolbar(orca_state.locusOfFocus) \
+                    or self.utilities.isFocusModeWidget(obj)
                 msg = "WEB: Updating focus and context to %s, %i" % (obj, offset)
                 debug.println(debug.LEVEL_INFO, msg, True)
                 orca.setLocusOfFocus(event, obj, notify)
@@ -1827,11 +1861,6 @@ class Script(default.Script):
             debug.println(debug.LEVEL_INFO, msg, True)
             return True
 
-        if event.source.getRole() == pyatspi.ROLE_FRAME:
-            msg = "WEB: Flusing messages from live region manager"
-            debug.println(debug.LEVEL_INFO, msg, True)
-            self.liveRegionManager.flushMessages()
-
         return True
 
     def onSelectedChanged(self, event):
@@ -1846,7 +1875,7 @@ class Script(default.Script):
             msg = "WEB: Event believed to be chrome page switch"
             debug.println(debug.LEVEL_INFO, msg, True)
             if event.detail1:
-                self.presentObject(event.source)
+                self.presentObject(event.source, priorObj=orca_state.locusOfFocus)
             return True
 
         if not self.utilities.inDocumentContent(event.source):
@@ -1943,6 +1972,12 @@ class Script(default.Script):
             return True
 
         obj, offset = self.utilities.getCaretContext(getZombieReplicant=False)
+        if obj and obj != event.source \
+           and not pyatspi.findAncestor(obj, lambda x: x == event.source):
+            msg = "WEB: Ignoring event because it isn't %s or its ancestor" % obj
+            debug.println(debug.LEVEL_INFO, msg, True)
+            return True
+
         if self.utilities.isZombie(obj):
             if self.utilities.isLink(obj):
                 msg = "WEB: Focused link deleted. Taking no further action."
